GLOBAL  __terminate, _abort
GLOBAL  __heapbase, __IsLoadedByBootsector, __FATBootDiskHint
EXTERN  _main, __atexitIterator

BootSectorSeg           equ     7C0h
bsDriveNumber           equ     24h     ; offset of boot drive # inside boot sector

; This module (c0?.obj) must be linked as the first module of your program

%ifdef  __TINY__
; This module is for tiny memory model, where CS = DS = ES = SS and the stack
; is in the end of the segment of the .COM program.
; Group all code/data segments together into a single physical segment:
GROUP DGROUP _TEXT _DATA _BSS _BSSEND
%else   ;%ifdef  __TINY__
GROUP DGROUP _DATA _BSS _BSSEND
%endif  ;%ifdef  __TINY__

; The code segment must be declared and placed first for a .COM program:
SEGMENT _TEXT PUBLIC CLASS=CODE USE16

%ifdef  __TINY__
        resb    100h            ; reserve 100h bytes for a normal .COM program
..start:
%else   ;%ifdef  __TINY__
..start:
; Init segment registers:
        mov     ax, _DATA
        mov     ds, ax
; Set the stack pointer to the end of the data segment
; (remember SS = DS for normal small model in Borland/Turbo C/C++),
; this makes things pretty much the same as with .COM files
; (e.g. tiny memory model, where CS = DS = ES = SS and the stack
; is in the end of the segment of the .COM program):
        cli
        mov     ss, ax
        xor     ax, ax
        mov     sp, ax
        sti
%endif  ;%ifdef  __TINY__

        call    _CheckIfBootsector

        push    ds
        pop     es

        call    _main
        call    __atexitIterator

        ; loop forever if started from bootsector:
        cmp     word [__IsLoadedByBootsector], 1
        je      $

__terminate:
_abort:
        mov     ax, 4c00h
        int     21h

_CheckIfBootsector:
; set __IsLoadedByBootsector to nonzero if loaded by bootsector
        sub     si, 16381 ; prime number 2**14-3
        mov     ax, si
        sbb     di, 32749 ; prime number 2**15-19
        or      ax, di
        sbb     bp, 65521 ; prime number 2**15-15
        or      ax, bp
        jz      .1

; then it should be DOS (or windows), flushing disk buffers:
        mov     word [__IsLoadedByBootsector], 0
        mov     ah, 0dh
        int     21h

; find the full program's path name after the environment:
        mov     es, [es:2ch] ; es = environment segment
        xor     bx, bx
.2:
        mov     cx, [es:bx]
        inc     bx
        inc     cx
        loop    .2
        xor     ax, ax
        mov     al, [es:bx+3] ; take the disk letter
        and     al, ~ ('a'-'A') ; make it uppercase
        sub     al, 'A'
        jmp     short .0

.1:
        push    BootSectorSeg
        pop     es
        mov     al, [es:bsDriveNumber]
        ; convert 0x80, 0x81, etc to 2, 3, etc
        cmp     al, 80h
        jc      .0
        sub     al, 80h-2
.0:
        mov     [__FATBootDiskHint], ax
        ret

; Declare empty data segments in this module:
SEGMENT _DATA PUBLIC CLASS=DATA
%ifndef __TINY__
; Make sure all other data are placed at non-zero offset:
        dd      "NULL"
%endif
__IsLoadedByBootsector  dw      1
__FATBootDiskHint       dw      -1

SEGMENT _BSS  PUBLIC CLASS=BSS

SEGMENT _BSSEND PUBLIC ALIGN=2 CLASS=BSS
__heapbase:

%ifdef  __TINY__
%else   ;%ifdef  __TINY__
; Allocate a small stack to avoid TLINK warnings:
SEGMENT _STACK STACK CLASS=STACK
        resb    512
%endif  ;%ifdef  __TINY__
