#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include "FAT.h"
#include "mydirent.h"

#include "frename.h"

#ifdef __cplusplus
extern "C" {
#endif
extern tFATFS stdio_FATFS;
extern int stdio_IsFATFSInit;
extern int stdio_Err2ErrNo (tIORes IORes);
#ifdef __cplusplus
}
#endif

typedef struct tFATDIR {
  tReadDirState ReadDirState;
  struct dirent de;
} tFATDIR;

DIR* opendir (
#if !defined(__TURBOC__)
const // improperly defined in Borland C/C++
#endif
char *dirname)
{
  tFATDIR* pFATDIR;
  tIORes IORes;

  if (!stdio_IsFATFSInit)
    return NULL;

  if ((dirname == NULL) || (!*dirname))
  {
    errno = EINVAL;
    return NULL;
  }

  pFATDIR = (tFATDIR*) malloc(sizeof(tFATDIR));
  if (pFATDIR == NULL)
  {
    errno = ENOMEM;
    return NULL;
  }

  if (!FAT_opendir (&stdio_FATFS, &IORes, dirname, &pFATDIR->ReadDirState))
  {
    free (pFATDIR);
    pFATDIR = NULL;
  }

  if (IORes != ior_OK)
    errno = stdio_Err2ErrNo (IORes);

  return (DIR*)pFATDIR;
}

int closedir (DIR *dir)
{
  tIORes IORes;

  if (!stdio_IsFATFSInit)
    return -1;

  if (dir == NULL)
  {
    errno = EINVAL;
    return -1;
  }

  FAT_closedir (&stdio_FATFS, &IORes, &((tFATDIR*)dir)->ReadDirState);
  free (dir);

  if (IORes != ior_OK)
  {
    errno = stdio_Err2ErrNo (IORes);
    return -1;
  }

  return 0;
}

void rewinddir (DIR *dir)
{
  tIORes IORes;

  if (!stdio_IsFATFSInit)
    return;

  if (dir == NULL)
  {
    errno = EINVAL;
    return;
  }

  FAT_rewinddir (&stdio_FATFS, &IORes, &((tFATDIR*)dir)->ReadDirState);

  if (IORes != ior_OK)
    errno = stdio_Err2ErrNo (IORes);
}

struct dirent* readdir (DIR *dir)
{
  tIORes IORes;

  tFATDirectoryEntry DirEntry;

  if (!stdio_IsFATFSInit)
    return NULL;

  if (dir == NULL)
  {
    errno = EINVAL;
    return NULL;
  }

  FAT_readdir (&stdio_FATFS, &IORes, &((tFATDIR*)dir)->ReadDirState, &DirEntry);
  DirEntryNameToASCIIZ (&DirEntry, ((tFATDIR*)dir)->de.d_name);

  if (IORes == ior_NO_MORE_FILES)
    return NULL;

  if (IORes != ior_OK)
  {
    errno = stdio_Err2ErrNo (IORes);
    return NULL;
  }

  return &((tFATDIR*)dir)->de;
}
